import java.util.*;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class Sbml_Sal_Translator extends DefaultHandler
{
  private ArrayList listOfCompartments = new ArrayList();
  private ArrayList listOfSpecies      = new ArrayList();
  private ArrayList listOfReactions    = new ArrayList();
  private ArrayList listOfReactants    = new ArrayList();
  private ArrayList listOfProducts     = new ArrayList();
//  private KineticLaw kineticLaw        = new KineticLaw();
  private ArrayList listOfParameters   = new ArrayList();
 
  private boolean isListOfReactants    = false;
  private boolean isListOfProducts     = false;
  private boolean isListOfParameters   = false;

  private static String reactionName;
  private static String reactionReversible;
  private static String formula;

  public static void main(String[] args) throws Exception
  {
    System.out.println("Here we go ...");
    Sbml_Sal_Translator readerObj = new Sbml_Sal_Translator();
    readerObj.read(args[0]);
  }

  public void read (String fileName) throws Exception
  {
    XMLReader readerObj = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");                readerObj.setContentHandler (this);
    readerObj.parse (fileName);
  }
 
  public void startDocument() throws SAXException
  {
    System.out.println("Starting ...");
  }
 
  public void endDocument() throws SAXException
  {
    System.out.println("... Finished");
    printCompartments();
  }

  public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
  {
    if (qName.equals("compartment"))
    { 
       listOfCompartments.add(new Compartment(atts.getValue("name"),atts.getValue("volume")));
    }
    
    if (qName.equals("specie"))
    {
       listOfSpecies.add(new Species(atts.getValue("name"),atts.getValue("initialAmount"),atts.getValue("compartment")));
    }
   
    if (qName.equals("reaction"))
    {
       reactionName = atts.getValue("name");
       reactionReversible = atts.getValue("reversible");
    }
    
    if (qName.equals("listOfReactants"))
    {
       isListOfReactants = true;
    }
    
    if (qName.equals("listOfProducts"))
    {
       isListOfProducts = true;
    }

    if (qName.equals("specieReference"))
    {
       if (isListOfReactants)
       { 
          listOfReactants.add(new SpecieReference(atts.getValue("specie"),atts.getValue("stoichiometry")));
       }
       else if(isListOfProducts)
       {
          listOfProducts.add(new SpecieReference(atts.getValue("specie"),atts.getValue("stoichiometry")));
       }
    }

    if (qName.equals("kineticLaw"))
    {
        kineticLaw.setFormula(atts.getValue("formula"));
    }
    
    if (qName.equals("listOfParameters"))
    {
       isListOfParameters = true;
    }

    if (qName.equals("parameter"));
    {
       listOfParameters.add(new ListOfParameters(atts.getValue("name"),atts.getValue("value")));
    }
  
  }
 
  public void endElement(String uri, String localName, String qName) throws SAXException
  {
   
    if (qName.equals("listOfReactants"))
    {
       isListOfReactants = false;
    }

    if (qName.equals("listOfProducts"))
    {
       isListOfProducts = false;
    }

    if (qName.equals("reaction"))
    {
        Reaction r = new Reaction(reactionName,reactionReversible);
        r.cloneListOfReactants(listOfReactants);
        r.cloneListOfProducts(listOfProducts);
        KineticLaw k = new KineticLaw(formula);
        k.cloneListOfParameters(listOfParameters);
        r.clone(k);
        listOfReactants.clear();
        listOfProducts.clear();
        listOfReactions.add(r);
    }
    
       
  }

  public void printCompartments()
  {
    for(int i=0; i<listOfCompartments.size(); i++)
    {
       Compartment c = (Compartment)listOfCompartments.get(i);
       System.out.println("Compartment name: " + c.getName()
                          + ", Volume: " + c.getVolume());
    }
    
    for(int i=0; i<listOfSpecies.size(); i++)
    {
       Species s = (Species)listOfSpecies.get(i);
       System.out.println("Species name: " + s.getName()
         + ", InitialAmount: " + s.getInitialAmount() + ", Compartment: " + s.getCompartment());
    }
      
    for(int i=0; i<listOfReactions.size();i++)
    {
       Reaction r = (Reaction)listOfReactions.get(i);
       System.out.println("Reaction name: " + r.getName()
         + ", Reversible: " +  r.getReversible());
       
       for (int j=0; j<r.getListOfReactantsSize();j++)
       {
           SpecieReference s = (SpecieReference) r.getListOfReactants().get(j);
           System.out.println("Reactant Name: " +  s.getSpecie() + ", Stoichiometry: " + s.getStoichiometry());
       }

       for (int j=0; j<r.getListOfProductsSize();j++)
       {
           SpecieReference s = (SpecieReference) r.getListOfProducts().get(j);
           System.out.println("Product Name: " +  s.getSpecie() + ", Stoichiometry: " + s.getStoichiometry());
       }
       
       System.out.println("Kinetic Law: " + r.getKineticLaw());
          
    }
  }

}

class Compartment
{
  public Compartment(String name, String volume)
  {
    this.name = name;
    this.volume = volume;
  }

  public String getName()
  {
    return name;
  }
  
  public String getVolume()
  {
    return volume;
  }
  
  private String name;
  private String volume;
} 


class Species
{
  public Species(String name, String initialAmount, String compartment)
  {
    this.name = name;
    this.initialAmount = initialAmount;
    this.compartment = compartment;
  }

  public String getName()
  {
    return name;
  }
  
  public String getInitialAmount()
  {
    return initialAmount;
  }
  
  public String getCompartment()
  {
    return compartment;
  }
  
  private String name;
  private String initialAmount;
  private String compartment;
}

class Reaction 
{
  public Reaction(String name, String reversible)
  {
    this.name = name;
    this.reversible = reversible;
  }

  public String getName()
  {
    return name;
  }
  
  public String getReversible()
  {
    return reversible;
  }

  public void cloneListOfReactants(ArrayList listOfReactants)
  {
     this.listOfReactants = (ArrayList) listOfReactants.clone();// Note: A Shallow Copy is enough in this case
                                                                //       since the contents of this Arraylist
                                                                //       are immutable.
  }
  
  public int getListOfReactantsSize()
  {
     return listOfReactants.size();
  }

  public ArrayList getListOfReactants()
  {
     return listOfReactants;
  }

  public void cloneListOfProducts(ArrayList listOfProducts)
  {
     this.listOfProducts = (ArrayList) listOfProducts.clone();// Note: A Shallow Copy is enough in this case
                                                              //       since the contents of this Arraylist
                                                              //       are immutable.
  }
  
  public int getListOfProductsSize()
  {
     return listOfProducts.size();
  }

  public ArrayList getListOfProducts()
  {
     return listOfProducts;
  }

  public void clone(String kineticLaw)
  {
     this.kineticLaw = kineticLaw;
  }

  public String getKineticLaw()
  {
     return kineticLaw;
  }

  private String name;
  private String reversible;
  private ArrayList listOfReactants = new ArrayList();
  private ArrayList listOfProducts  = new ArrayList();
  private KineticLaw kineticLaw     = new KineticLaw();
}

class SpecieReference
{
  public SpecieReference(String specie, String stoichiometry)
  {
    this.specie = specie;
    this.stoichiometry = stoichiometry;
  }

  public String getSpecie()
  {
    return specie;
  }
  
  public String getStoichiometry()
  {
    return stoichiometry;
  }
  
  private String specie;
  private String stoichiometry;
}

class KineticLaw
{
  public KineticLaw(String formula)
  {
    this.formula = formula;
  }

  public void cloneListOfParameters(ArrayList listOfParameters)
  {
     this.listOfParameters = (ArrayList) listOfParameters.clone();// Note: A Shallow Copy is enough in this case
                                                                //       since the contents of this Arraylist
                                                                //       are immutable.
  }
  
  public int getListOfParametersSize()
  {
     return listOfParameters.size();
  }

  public ArrayList getListOfParameters()
  {
     return listOfParameters;
  }

  private String kineticLaw;
  private ArrayList listOfParameters = new ArrayList();
}
