// package sbmlClasses;
/**
 * A <code>Compartment</code> object represents a bounded volume in which species are located.
 */
// HISTORY:
// Sep23: Added "name" to the constructor formal arguments.
// Sep23: if id=null, then id=name.
// Sep24: Adding "full SAL module" information in Compartment Class.

import java.io.*;
import java.util.*;

public class Compartment
{
  public Compartment(String id, String spatialDimensions, String name)
  {
    this.id = id;
    this.name = name;
    this.spatialDimensions = spatialDimensions;
    if (id.equals("null")) this.id = name;
  }
  /**
   * Gets the Id of the Compartment.
   * @return id  The Identifier of the Compartment.
   */
  public String getId()
  {
    return id;
  }

  /**
   * Gets the spatialDimensions of the Compartment.
   * @return spatialDimensions A value of "3" means a three dimensional structure (a volume).
   *                                      "2"         two   dimensional structure (an area).
   *                                      "1"         one   dimensional structure (a line). 
   */  
  public String getSpatialDimensions()
  {
    return spatialDimensions;
  }
  
  /** ASHISH */
  public void addInputSpecies(String name)
  {
    boolean add = true;
    for (int i = 0; i < inputSpecies.size(); i++) {
	if (name.equals((String)inputSpecies.get(i))) {
	   add = false;
	   break;
	}
    }
    if (!add) return;
    for (int i = 0; i < localSpecies.size(); i++) {
	if (name.equals(((Species)localSpecies.get(i)).getId())) {
	   add = false;
	   break;
	}
    }
    if (add) inputSpecies.add(name);
  }

  public void setLocalSpecies(ArrayList allSpecies, ArrayList listOfRateRules)
  {
    if (allSpecies != null)
       localSpecies = (ArrayList) allSpecies.clone();
    boolean generateDotVariable = false;
    for (int j=0; j<localSpecies.size(); j++)               // Generates a dotVariable for the species in Local
    {                                                             // species list array if and only iff the constant
        Species species = (Species)localSpecies.get(j);     // attribute for the species is "false".
             
        if ( species.getConstant().equals("true") )
        {
                generateDotVariable = false;
        }
        else if ( species.getBoundaryCondition().equals("true") )  // Generates a dotVariable for species in Local species
        {                                                          // list array even if the BoundaryCondition attribute is 
                  generateDotVariable = false;                          // "true", if in case there exists a rateRule for the 
                  for (int k=0; k<listOfRateRules.size(); k++)          // Variable.
                  {
                      Rule l = (Rule) listOfRateRules.get(k);
                      if ( l.getVariable().equals( species.getId() ) )
                      {
                         generateDotVariable = true;
                      }
                  }
         }
         else
         {
                  generateDotVariable = true;
         }
             
         if (generateDotVariable)
         {
                localDotSpecies.add( species );
         }
     }

  }

  public void setLocalReactions(ArrayList reactions)
  {
    if (reactions != null)
    localReactions = (ArrayList) reactions.clone();
  }

  public void setLocalParameters(ArrayList parameters)
  {
    if (parameters != null)
    localParameters = (ArrayList) parameters.clone();
  }

  /** Mark this parameter as a "chaning" param, not a "static" one. */
  public void markParameter(Parameter param)
  {
    if (localDummyParameters == null)
	localDummyParameters = new ArrayList();
    for (int i = 0; i < localParameters.size(); i++) {
	Parameter lparam = (Parameter) localParameters.get(i);
	if (((String)lparam.getId()).equals((String) param.getId())) {
	   localDummyParameters.add(lparam);
	   localParameters.remove(i);
	   return;
	}
    }
  }

  public void setLocalAssignmentRules(ArrayList assignmentRules)
  {
    if (assignmentRules != null)
    localAssignmentRules = (ArrayList) assignmentRules.clone();
  }

  public void setLocalAlgebraicRules(ArrayList algebraicRules)
  {
    if (algebraicRules != null)
    localAlgebraicRules = (ArrayList) algebraicRules.clone();
  }

  public void setLocalRateRules(ArrayList rateRules)
  {
    if (rateRules != null)
    localRateRules = (ArrayList) rateRules.clone();
  }

  public void setLocalEvents(ArrayList events)
  {
    if (events != null)
    localEvents = (ArrayList) events.clone();
  }

  public void setLocalPEvents(ArrayList events)
  {
    if (events != null) {
        localPEvents = (ArrayList) events.clone();
	System.out.println("Compartment: " + id + ", Added PEvents of size " + String.valueOf(events.size()));
    }
  }

  public String getCall()
  {
    if (inputSpecies.size() == 0) return(null);
    StringBuffer ret = new StringBuffer();
    ret.append(this.getId());
    if  (localParameters.size() != 0) {
	ret.append("[0");
        for (int j=1; j<localParameters.size(); j++)
	    ret.append(", 0");
	ret.append("]");
    }
    return(ret.toString());
  }

  public void toSalFile(PrintWriter out, ArrayList listOfUniqueGlobalParameters)
  {
	 if (inputSpecies.size() == 0) return;
         out.print(id);

         if (localParameters.size() != 0) {
            out.print("[");
            for (int j=0; j<localParameters.size(); j++) {
                Parameter parameter = (Parameter) localParameters.get(j);
                if (j != 0) out.print(", ");
                out.print( parameter.getId() );
            }
            out.print(" : REAL]");
         }
        
         out.println(" : MODULE =");

         out.println("\t" + "BEGIN");

         if (listOfUniqueGlobalParameters.size() > 0) 
	    out.print("\t" + "INPUT ");
         for (int j=0; j<listOfUniqueGlobalParameters.size(); j++)
         {
             Parameter parameter = (Parameter) listOfUniqueGlobalParameters.get(j);
             out.print(parameter.getId());

             if (j < listOfUniqueGlobalParameters.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }
        
	 /** input variable declaration */
         if (inputSpecies.size() > 0) out.print("\t" + "INPUT ");
         for (int j=0; j<inputSpecies.size(); j++)
         {
             String species = (String)inputSpecies.get(j);
             out.print(species);

             if (j < inputSpecies.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }

	 /** local variable declaration */
         out.print("\t" + "OUTPUT ");
         for (int j=0; j<localSpecies.size(); j++)
         {
             Species species = (Species)localSpecies.get(j);
             out.print(species.toString());

             if (j < localSpecies.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }

	 /** dot variables declaration */
         out.print("\t" + "LOCAL ");
         for (int j=0; j<localDotSpecies.size(); j++)
         {
             Species species = (Species)localDotSpecies.get(j);
             out.print(species.toString() + "dot");

             if (j < localDotSpecies.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }

	 /** Parameters that change are declared as LOCAL variables. */
         if ((localDummyParameters != null) && (localDummyParameters.size() > 0)) {
            out.print("\t" + "LOCAL ");
            for (int j=0; j<localDummyParameters.size(); j++) {
                Parameter parameter = (Parameter) localDummyParameters.get(j);
		if (j != 0) out.print(", ");
             	out.print(parameter.getId());
	    }
            out.println(" : REAL");
	}

        if ( localParameters.size() > 0 || localAssignmentRules.size() > 0 || localAlgebraicRules.size() > 0 )
        {
           out.println("\tINVARIANT");                                                
           out.print("\t       ");

           for (int j=0; j<localParameters.size(); j++)
           {
               Parameter parameter = (Parameter) localParameters.get(j);
               out.print( parameter.getId() + " = " + parameter.getValue() );
               if (j < localParameters.size()-1 || localAssignmentRules.size() > 0 ||
                       localAlgebraicRules.size() > 0 )
               {
                  out.print(" AND ");
               }
               else
               {
                  out.println("");
               }
           }

           for (int j=0; j<localAssignmentRules.size(); j++)
           {
               Rule localAssignmentRule = (Rule) localAssignmentRules.get(j);

               out.print( localAssignmentRule.getVariable() + " = " + localAssignmentRule.getExpression() );

               if ( j < localAssignmentRules.size()-1 || localAlgebraicRules.size() > 0 )
               {
                  out.println(" AND");
               }
               else
               {
                  out.println();
               }
           }

           for (int j=0; j<localAlgebraicRules.size(); j++)
           {
               Rule localAlgebraicRule = (Rule) localAlgebraicRules.get(j);

               out.print( localAlgebraicRule.getExpression() + " = 0" );

               if ( j < localAlgebraicRules.size()-1 )
               {
                  out.println(" AND");
               }
               else
               {
                  out.println();
               }
           }

        } /* Finished writing INVARIANT section for the current Module */ 

        if (localSpecies.size() != 0)
        {
           out.println("\tINITIALIZATION");
           out.print("\t       ");

           for (int j=0; j<localSpecies.size(); j++)
           {
               Species lSpecies = (Species) localSpecies.get(j);
               out.print( lSpecies.toString() + " = " + lSpecies.getInitialAmount() );
               if ( j < localSpecies.size()-1 ) 
               {
                  out.print("; ");
               }
               else 
               {
                  out.println("");
               }
           }

        } /* Finished writing INITIALIZATION section for the current Module */

        if (localSpecies.size() != 0)
        {
            boolean rateRuleVariable = false;
            out.println("\tTRANSITION");
           
	    int j = 0;
	    if (localPEvents.size() <= 0) 
		System.out.println("Compartment: " + id + ", localPEvents size is " + String.valueOf(localPEvents.size()));
	    while (j < localPEvents.size())
            { 
               Event firstEvent = (Event) localPEvents.get(j);
	       String eventParam = (String) firstEvent.getListOfEventVariables().get(0);
	       boolean first = true;
               out.print( "\t\t" + eventParam + " = ");
	       while (j < localPEvents.size() && ((String) ((Event)localPEvents.get(j)).getListOfEventVariables().get(0)).equals(eventParam))
               {
                   Event localEvent = (Event) localPEvents.get(j);
		   if (first) out.print("IF ");
		   else out.print("\t\t\tELSIF ");
		   first = false;
                   out.println(localEvent.getTriggerExpression() );
                   out.print( "\t\t\tTHEN ");
		   String rhs = (String)localEvent.getListOfEventAssignments().get(0); 
		   int ind = rhs.indexOf('=');
                   out.println(rhs.substring(ind+1));
		   j++;
               }
               out.println( "\t\t\tELSE 0 ENDIF; ");
            }
	       
            if (localEvents.size() !=0)
            { 
               out.println("\t       [");

               for (j=0; j<localEvents.size(); j++ )
               {
                   Event localEvent = (Event) localEvents.get(j);
                   out.print( "\t       " + localEvent.getTriggerExpression() );
                   out.println("-->");
                   
                   for (int k=0; k<localEvent.getListOfEventAssignmentsSize(); k++)
                   {
                       out.print( "\t\t       " + localEvent.getListOfEventAssignments().get(k) ); 
              
                       if (k < localEvent.getListOfEventAssignmentsSize() - 1)
                       {
                          out.println(";");
                       }
                       else
                       {
                          out.println();
                       }
                   }
                  
                   if (j < localEvents.size()-1)
                   {
                      out.println("\t       []");
                   }
                   else
                   {
                      out.println("\t       ]");
                   }
               }
            }

           /* This block writes the rate reactions determining the change in species concentration.      */
           /* For each dotSpecies we need to check whether its rate is determined by Rate Rule before    */
           /* generating a differential equation for that species. If the rate is not determined by      */
           /* Rate Rule then build a differential equation for the species with help of Kinetic formulas */
           /* of the Reactions in which the species appears. If the species appears in list of Reactants */
           /* then place a '-' sign before the Kinetic formula else if the species appears in list of    */
           /* products then place '+' sign before the Kinetic formula.                                   */

	   if (localPEvents.size() != 0) {
               out.println("\t\t[ TRUE -->");
           for (j=0; j<localDotSpecies.size(); j++)
           {
               Species dotSpecies       = (Species) localDotSpecies.get(j);
                       rateRuleVariable = false;

               for (int l=0; l<localRateRules.size(); l++)
               {
                   Rule localRateRule = (Rule) localRateRules.get(l);
                   if ( dotSpecies.getId().equals( localRateRule.getVariable() ) )
                   {
                      rateRuleVariable = true;
                      out.print("\t       ");
                      out.print( localRateRule.getVariable() + "dot' = " + localRateRule.getExpression() );

                      if ( j < localDotSpecies.size()-1 )
                      {
                         out.println(" ;");
                      }
                      else
                      {
                         out.println();
                      }

                      break;
                   }
               } /* Finished writing all the Rate Rules */
               
	       /* Generating Differential Equations. */
               if (!rateRuleVariable)                              // Generate differential equations for species
               {                                                   // whose rate change is not determined by Rate Rule.
                  out.print("\t       ");
                  out.print( dotSpecies.toString() + "dot'" + " = " );
                  boolean firstProductExpression = true;

                  for (int k=0; k<localReactions.size(); k++)
                  {
                      Reaction localReaction = (Reaction)   localReactions.get(k);
                      KineticLaw kineticLaw  = (KineticLaw) localReaction.getKineticLaw();
                   
                      for (int l=0; l<localReaction.getListOfReactantsSize();l++)
                      {
                  
                          SpeciesReference speciesRef = (SpeciesReference) localReaction.getListOfReactants().get(l);
                   
                          if (dotSpecies.getId().equals( speciesRef.getSpecies() ) )
                          {                                                                  
                             out.print( " - " + speciesRef.getStoichiometry() + "*" + kineticLaw.getFormula() );
                                                                                      // If a dotSpecies appears in list of
                                                                                      // reactants, then apply a negative   
                             firstProductExpression = false;                          // sign to kinetic formula.
                             break;
                          }
                   
                      }
                  
                      for (int l=0; l<localReaction.getListOfProductsSize();l++)
                      {
                          SpeciesReference speciesRef = (SpeciesReference) localReaction.getListOfProducts().get(l);
                     
                          if ( dotSpecies.getId().equals( speciesRef.getSpecies() ) )
                          {
                             if (firstProductExpression)                            // Do not print '+' before kineticFormula
                             {                                                      // if the kineticFormula is the first one. 
                                out.print( " " + speciesRef.getStoichiometry() + "*" + kineticLaw.getFormula() );          
                                firstProductExpression = false;
                             }
                             else
                             {
                                out.print( " + " + speciesRef.getStoichiometry() + "*" + kineticLaw.getFormula() );
                             }
                             break;
                          }
                       }
                  }
                  
                  if ( j < localDotSpecies.size()-1 )
                  {
                     out.println(" ;");
                  }
                  else
                  {
                     out.println();
                  }
  
               }
           }
	   //if (localPEvents.size() != 0)
               out.println("\t\t]");
	   }

           out.println("\tEND;\n\n");
        }

  }

  private String id;
  private String name;
  private String spatialDimensions;
  private ArrayList localSpecies;
  private ArrayList inputSpecies = new ArrayList();
  private ArrayList localDotSpecies = new ArrayList();
  private ArrayList localReactions;
  private ArrayList localParameters;		// module[a, b]: Constant params
  private ArrayList localDummyParameters;	// Inside-Out-Representation: Changing params
  private ArrayList localAssignmentRules;
  private ArrayList localAlgebraicRules;
  private ArrayList localRateRules;
  private ArrayList localEvents;
  private ArrayList localPEvents;
} 
