// package sbmlClasses;
// History:
// Sep17: added checks in the constructor.

import java.util.*;

/**
 * An <code>Event</code> object stores the information about when an event can occur, the variables
 * that are affected by the event, and how the variables are affected.
 */ 
public class Event
{
  public Event(String triggerExpression, ArrayList listOfEventAssignments, 
		ArrayList listOfEventVariables, ArrayList listOfTriggerVariables)
  {
    this.triggerExpression      = triggerExpression;
    if (listOfEventAssignments == null)
       this.listOfEventAssignments = null;
    else 
       this.listOfEventAssignments = (ArrayList) listOfEventAssignments.clone();
    if (listOfEventVariables == null)
       this.listOfEventVariables   = null;
    else
       this.listOfEventVariables   = (ArrayList) listOfEventVariables.clone();
    if (listOfTriggerVariables == null)
       this.listOfTriggerVariables   = null;
    else
       this.listOfTriggerVariables   = (ArrayList) listOfTriggerVariables.clone();
  }

  /** Gets the triggerExpression corresponding to the event object.
   *  @return triggerExpression - a boolean expression which evaluates to true when the event is fired. 
   */
  public String getTriggerExpression()
  {
    return this.triggerExpression;
  }

  /** Gets the size of ArrayList containing EventAssignments.
   *  @return size_of_EventAssignments_ArrayList
   */
  public int getListOfEventAssignmentsSize()
  {
    return this.listOfEventAssignments.size();
  }

  /** Gets the ArrayList containing EventAssignments.
   *  @return EventAssignments_ArrayList
   */
  public ArrayList getListOfEventAssignments()
  {
    return this.listOfEventAssignments;
  }

  /** Gets the size of Arraylist containing variables(Species) involved in defining the Event.
   *  @return size_of_EventVariables_ArrayList
   */
  public int getListOfEventVariablesSize()
  {
    return this.listOfEventVariables.size();
  }

  /** Gets the Arraylist containing variables(Species) involved in defining the Event.
   *  @return EventVariables_ArrayList
   */
  public ArrayList getListOfEventVariables()
  {
    return this.listOfEventVariables;
  }

  //ASHISH:
  public int getListOfTriggerVariablesSize()
  {
    return this.listOfTriggerVariables.size();
  }

  public ArrayList getListOfTriggerVariables()
  {
    return this.listOfTriggerVariables;
  }

  private String    triggerExpression;
  private ArrayList listOfEventAssignments;
  private ArrayList listOfEventVariables;
  private ArrayList listOfTriggerVariables;

}
