// package sbmlClasses;
import java.util.*;
/**
 * A <code>KineticLaw</code> object holds the information that describes the rate at which a reaction
 * takes place.
 */
public class KineticLaw implements Cloneable
{
  /**
   * This is a noparamter constructor which does nothing. This can be used to instantiate an object of the
   * class KineticLaw
   */
  public KineticLaw()
  {
                     // A do nothing default constructor
  }

  /**
   * This constructor initializes the formula field of the <code>KineticLaw</code> object.
   * @param formula kinetic formula which depicts the rate at which the reaction proceeds.
   */
  public KineticLaw(String formula)
  {
    this.formula = formula;
  }
  
  /**
   * This method makes a deep copy of the Kinetic Law object. Deep copy is necessary in this case because
   * <code>KineticLaw</code> obtains contains mutable feilds.
   */
  public Object clone() 
  {                    
     try
     {  
        //call Object.clone()
        KineticLaw cloned = (KineticLaw) super.clone();
        
        //clone mutable fields
        cloned.listOfParameters = (ArrayList) listOfParameters.clone();
        return cloned;
     }
     catch (CloneNotSupportedException e) 
     {
           return null;
     }
  }
  
  /**
   * This method makes a copy of the parameters stored in temporary ArrayList to the parameters ArrayList
   * in KineticLaw object.
   * @param listOfParamaeters A temporary ArrayList containing the list of parameter structures.
   */    
  public void cloneListOfParameters(ArrayList listOfParameters)
  {
     this.listOfParameters = (ArrayList) listOfParameters.clone();
  }
 
  /**
   * This method fetches the formula which has the  kinetic information for a reaction.
   * @return formula kinetic formula describing the rate at which a reaction takes place.
   */
  public String getFormula()
  {
     return formula;
  }

  /**
   * This method fetches the total number of <code>parameter</code> objects that are used in
   * kinetic formula to describe the rate of a reaction.
   * @return int size of listOfParameters ArrayList.
   */
  public int getListOfParametersSize()
  {
     return listOfParameters.size();
  }
  
  /**
   * This method fetches the reference to the ArrayList which contains a list of <code>parameter</code>
   * objects used in kinetic formula.
   * @return listOfParameters ArrayList containing a list of parameter objects used in kinetic formula.
   */
  public ArrayList getListOfParameters()
  {
     return listOfParameters;
  }

  private String formula;
  private ArrayList listOfParameters = new ArrayList();
}

