/**
 * Program Description: This purpose of this program is to translate a <code>SBML</code> file
 *                      to <code>SAL</code> file.
 * @author:             Sri Paladugu.
 * @version:            0.0a.
 * Date Compiled:       07/15/2003.
 * Input:               SBML File.
 * Output:              SAL File
 */

import sbmlClasses.*;
import java.io.*;
import java.util.*;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

/**
 * A <code>Sbml_Sal_Translator</code> object represents a SBML to SAL Translator. It extends
 * DefaultHandler which is an empty implementation of ContentHandler interface. ContentHanler
 * is a SAX interface which will be used by the xml parser to communicate with the object
 * Sbml_Sal_Translator.
 */   
public class Sbml_Sal_Translator extends DefaultHandler
{
  private ArrayList listOfCompartments        = new ArrayList();
  private ArrayList listOfSpecies             = new ArrayList();
  private ArrayList listOfReactions           = new ArrayList();
  private ArrayList listOfReactants           = new ArrayList();
  private ArrayList listOfProducts            = new ArrayList();
  private ArrayList listOfModifiers           = new ArrayList();
  private ArrayList listOfParameters          = new ArrayList();
  private ArrayList listOfGlobalParameters    = new ArrayList();
  private ArrayList listOfAlgebraicRules      = new ArrayList();
  private ArrayList listOfAssignmentRules     = new ArrayList();
  private ArrayList listOfRateRules           = new ArrayList();
  private ArrayList listOfFunctionDefinitions = new ArrayList();
  private ArrayList listOfFunctionVariables   = new ArrayList();
  private ArrayList listOfEvents              = new ArrayList();
  private ArrayList listOfEventAssignments    = new ArrayList();
  private ArrayList listOfEventVariables      = new ArrayList();
  private ArrayList listOfRuleVariables       = new ArrayList();

  private boolean isListOfReactants           = false;
  private boolean isListOfProducts            = false;
  private boolean isListOfModifiers           = false;
  private boolean isKineticLaw                = false;
  private boolean isMath                      = false;
  private boolean isMathIdentifier            = false;
  private boolean isSpeciesReference          = false;
  private boolean isAssignmentRule            = false;
  private boolean isAlgebraicRule             = false;
  private boolean isRateRule                  = false;
  private boolean isFunctionDefinition        = false;
  private boolean isFunctionVariable          = false;
  private boolean isEventAssignment           = false;
  private boolean isTrigger                   = false;

  private static String modelId;
  private static String reactionName;
  private static String reactionReversible;
  private static String formula;
  private static String speciesValue;
  private static String stoichiometryValue;
  private static String ruleVariable;
  private static String functionId;
  private static String functionVariable;
  private static String eventAssignment;
  private static String eventVariable;
  private static String triggerExpression;

  private static StringBuffer mathIdentifier = new StringBuffer();

  public static void main(String[] args) throws Exception
  {
    System.out.println("Here we go ...");
    Sbml_Sal_Translator readerObj = new Sbml_Sal_Translator();
    readerObj.read(args[0]);
  }

  /** 
   * This method provides the SAX Parser with the document to be parsed, by invoking parse() method.
   * @param fileName     The file to be parsed
   */
  public void read (String fileName) throws Exception
  {
    XMLReader readerObj = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");               
    readerObj.setContentHandler (this);
    readerObj.parse (fileName);
  }
  
  /**
   * This method catches <em>start of document<em> event.
   * @throws SAXException
   */
  public void startDocument() throws SAXException
  {
    System.out.println("Started Reading ...");
  }
 
  /**
   * This method catches <em>end of document<em> event.
   * @throws SAXException
   */
  public void endDocument() throws SAXException
  {
    printCompartments();
    System.out.println("... Finished Reading");
    try
    {
       writeToSalFile();
    }
    catch (Exception e)
    {
      System.err.println("Unable to write to Sal File");
      e.printStackTrace();
    }
  }

  /**
   * This method creates a file with .sal extension and writes the data in sal Format
   * @throws Exception
   */
  public void writeToSalFile() throws Exception
  {
     PrintWriter out = new PrintWriter (new FileWriter(modelId + ".sal"), true);
     out.print(modelId);                                                         // Begin of writing CONTEXT
     out.println(" : CONTEXT =");          
     out.println("BEGIN");                                                
   
     if (listOfGlobalParameters.size() != 0)
     {
        out.println("ENVIRONMENT : MODULE =");
        out.println("\tBEGIN");                                                
        out.print("\tOUTPUT "); 

        for (int j=0; j<listOfGlobalParameters.size(); j++)
        {
            Parameter parameter = (Parameter)  listOfGlobalParameters.get(j);
            out.print( parameter.getId() );
            if (j < listOfGlobalParameters.size()-1)
            {
               out.print(", ");
            }
            else
            {
               out.println(" : REAL");
            }
        }
     }

     ArrayList listOfGlobalAssignmentRules       = new ArrayList();
     ArrayList listOfGlobalAlgebraicRules        = new ArrayList();
     ArrayList listOfGlobalRateRules             = new ArrayList();
     ArrayList listOfConstantGlobalParameters    = new ArrayList();
     ArrayList listOfNonConstantGlobalParameters = new ArrayList();
     boolean   isGlobalRule                      = true;
     boolean   isGlobalParameter                 = false;
     String    ruleVariable                      = "";
     
     /** Get all the rules (Assignment/Algerbraic/Rate) which define the constraints on the Global Parameters into
         temporary list Arrays.
     */
     if (listOfGlobalParameters.size() != 0)                        
     {
        for (int j=0; j<listOfAssignmentRules.size(); j++)
        {
            Rule assignmentRule = (Rule) listOfAssignmentRules.get(j);
            isGlobalRule        = true;
          
            for (int k=0; k<assignmentRule.getListOfRuleVariablesSize(); k++)
            {
                isGlobalParameter = false;
                ruleVariable    = (String) assignmentRule.getListOfRuleVariables().get(k); 
                for (int l=0; l<listOfGlobalParameters.size(); l++)
                {
                    Parameter parameter = (Parameter) listOfGlobalParameters.get(l);
                    if ( ruleVariable.equals( parameter.getId() ) )
                    {
                       isGlobalParameter = true;
                    }
                }
                if (!isGlobalParameter)
                {
                   isGlobalRule = false;
                   break;
                }
            }
            
            if (isGlobalRule)
            {
               listOfGlobalAssignmentRules.add(assignmentRule);
            }
        }
      
        for (int j=0; j<listOfAlgebraicRules.size(); j++)
        {
            Rule algebraicRule  = (Rule) listOfAlgebraicRules.get(j);
            isGlobalRule        = true;
          
            for (int k=0; k<algebraicRule.getListOfRuleVariablesSize(); k++)
            {
                isGlobalParameter = false;
                ruleVariable    = (String) algebraicRule.getListOfRuleVariables().get(k); 
                for (int l=0; l<listOfGlobalParameters.size(); l++)
                {
                    Parameter parameter = (Parameter) listOfGlobalParameters.get(l);
                    if ( ruleVariable.equals( parameter.getId() ) )
                    {
                       isGlobalParameter = true;
                    }
                }
                if (!isGlobalParameter)
                {
                   isGlobalRule = false;
                   break;
                }
            }
            
            if (isGlobalRule)
            {
               listOfGlobalAlgebraicRules.add(algebraicRule);
            }
        }

        for (int j=0; j<listOfRateRules.size(); j++)
        {
            Rule rateRule = (Rule) listOfRateRules.get(j);
            isGlobalRule  = true;
          
            for (int k=0; k<rateRule.getListOfRuleVariablesSize(); k++)
            {
                isGlobalParameter = false;
                ruleVariable      = (String) rateRule.getListOfRuleVariables().get(k); 
                for (int l=0; l<listOfGlobalParameters.size(); l++)
                {
                    Parameter parameter = (Parameter) listOfGlobalParameters.get(l);
                    if ( ruleVariable.equals( parameter.getId() ) )
                    {
                       isGlobalParameter = true;
                    }
                }
                if (!isGlobalParameter)
                {
                   isGlobalRule = false;
                   break;
                }
            }
            
            if (isGlobalRule)
            {
               listOfGlobalRateRules.add(rateRule);
            }
        }

        if (listOfGlobalRateRules.size() != 0)
        {
           out.print("\tLOCAL ");
           for (int j=0; j<listOfGlobalRateRules.size(); j++)
           {
               Rule globalRateRule = (Rule) listOfGlobalRateRules.get(j);

               out.print( globalRateRule.getVariable() + "dot");

               if ( j < listOfGlobalRateRules.size()-1 )
               {
                  out.print(" ,");
               }
               else
               {
                  out.println(" : REAL");
               }
           }
        }
        
        /* Get the list of Global Parameters which will remain constant. */
        /* List of Constant Global Parameters are those for which there  */
        /* is no Global Rate Rules.                                      */

        boolean   isConstantGlobalParameter = true;

        for (int j=0; j<listOfGlobalParameters.size(); j++)
        {
            Parameter parameter = (Parameter) listOfGlobalParameters.get(j);
       
            for (int k=0; k<listOfGlobalRateRules.size(); k++)
            {
                Rule globalRateRule = (Rule) listOfGlobalRateRules.get(k);

                if ( globalRateRule.getVariable().equals(parameter.getId()) )
                {
                   isConstantGlobalParameter = false;
                   listOfNonConstantGlobalParameters.add(parameter);
                   break;      
                }
            }
           
            if (isConstantGlobalParameter) 
            {
               listOfConstantGlobalParameters.add(parameter);
            }
        }

        if ( listOfConstantGlobalParameters.size() > 0 )
        {
           out.println("\tINVARIANT");                                                
           out.print("\t       ");

           for (int j=0; j<listOfConstantGlobalParameters.size(); j++)
           {
               Parameter constantParameter = (Parameter) listOfConstantGlobalParameters.get(j);
               out.print( constantParameter.getId() + " = " + constantParameter.getValue() );

               if ( j < listOfConstantGlobalParameters.size()-1 || listOfGlobalAssignmentRules.size() > 0 ||
                    listOfGlobalAlgebraicRules.size() > 0 )
               {
                  out.print(" AND ");
               }
               else
               {
                  out.println();
               }
           }
 
        }
         
     } /* End of Global INVARIANT Section */

     if (listOfNonConstantGlobalParameters.size() != 0)
     {
        out.println("\tINITIALIZATION");
        out.print("\t       ");
        for (int j=0; j<listOfNonConstantGlobalParameters.size(); j++)
        {
            Parameter nonConstantGlobalParameter = (Parameter) listOfNonConstantGlobalParameters.get(j);
            out.print( nonConstantGlobalParameter.getId() + " = " + nonConstantGlobalParameter.getValue() );
            if (j < listOfNonConstantGlobalParameters.size()-1)
            {
               out.print("; ");
            }
            else
            {
               out.println("");
            }
        }
     } /* End of Global INITIALIZATION Section */

     if (listOfGlobalParameters.size() != 0)
     {
        out.println("\tTRANSITION");
        out.print("\t       ");
        for (int j=0; j<listOfGlobalRateRules.size(); j++)
        {
            Rule globalRateRule = (Rule) listOfGlobalRateRules.get(j);

            out.print( globalRateRule.getVariable() + "dot' = " + globalRateRule.getExpression() );

            if ( j < listOfGlobalRateRules.size()-1 )
            {
               out.println(";");
            }
            else
            {
               out.println();
            }
        }
        out.println("\tEND");
     } /* End of Global TRANSITION Section */

     /* Begin Writing Function Definitions */
     for (int i=0; i<listOfFunctionDefinitions.size(); i++)
     {  
         FunctionDefinition f = (FunctionDefinition)  listOfFunctionDefinitions.get(i);
         out.print(f.getFunctionId() + "(" );
         
         for (int j=0; j<f.getListOfFunctionVariablesSize(); j++)
         {
             out.print(f.getListOfFunctionVariables().get(j) );

             if (j < f.getListOfFunctionVariablesSize()-1)
             {
                out.print(",");
             }
             else
             {
                out.print(": REAL)");
             }
         }
         out.println(" : REAL = ");

         out.println("\t" + f.getFunctionExpression() + ";");
     } /* End of Function Definitions section */

     ArrayList listOfLocalSpecies    = new ArrayList();
     ArrayList listOfLocalReactions  = new ArrayList();
     boolean   reactionAdded         = false;

     for (int i=0; i<listOfCompartments.size(); i++)                            // Begin of writing MODULES
     {
         Compartment compartment = (Compartment)listOfCompartments.get(i);
         out.print(compartment.getId());
         
         for (int j=0; j<listOfSpecies.size(); j++)                            // Get all the species corresponding to the
         {                                                                     // Compartment/Module.
       	     Species species = (Species)listOfSpecies.get(j);

             if ( species.getCompartment().equals( compartment.getId() ) )
             {
                listOfLocalSpecies.add(species); 
             }
         }

         for (int j=0; j<listOfReactions.size(); j++)                            // Get all the reactions corresponding to the 
         {                                                                       // Compartment/Module to a local List Array.
             Reaction reaction              = (Reaction)listOfReactions.get(j);
                      reactionAdded         = false;
             
             /** If a reactant-species matches a species in Local Species list Array then the reaction belongs to local
                 Compartment/Module */
             for (int k=0; k<reaction.getListOfReactantsSize() && (!reactionAdded); k++)
             {
                 SpeciesReference speciesRef = (SpeciesReference) reaction.getListOfReactants().get(k);
               
                 for (int l=0; l<listOfLocalSpecies.size() && (!reactionAdded); l++)
                 {
                     Species localSpecies = (Species) listOfLocalSpecies.get(l);
                     if ( localSpecies.getId().equals(speciesRef.getSpecies()) )
                     {
                        listOfLocalReactions.add(reaction);
                        reactionAdded = true;
                     }
                 }                                                                          
           
             }
            
             /** If a product-species matches a species in Local Species list Array then the reaction belongs to local
                 Compartment/Module. Note: Search the Product List Array only if the reaction is not already added to 
                 Local reactions list Array */
             for (int k=0; k<reaction.getListOfProductsSize() && (!reactionAdded); k++)
             {
                 SpeciesReference speciesRef = (SpeciesReference) reaction.getListOfProducts().get(k);
              
                 for (int l=0; l<listOfLocalSpecies.size() && (!reactionAdded); l++)
                 {
                     Species localSpecies = (Species) listOfLocalSpecies.get(l);
                     if ( localSpecies.getId().equals(speciesRef.getSpecies()) )
                     {
                        listOfLocalReactions.add(reaction);
                        reactionAdded = true;
                     }
                 }                                                                          
              
             }
             /** If a modifier-species matches a species in Local Species list Array then the reaction belongs to local
                 Compartment/Module. Note: Search the Modifier List Array only if the reaction is not already added to 
                 Local reactions list Array */
               
             for (int k=0; k<reaction.getListOfModifiersSize() && (!reactionAdded); k++)
             {
                 ModifierSpeciesReference modifierRef = (ModifierSpeciesReference) reaction.getListOfModifiers().get(k);
              
                 for (int l=0; l<listOfLocalSpecies.size() && (!reactionAdded); l++)
                 {
                     Species localSpecies = (Species) listOfLocalSpecies.get(l);
                     if ( localSpecies.getId().equals(modifierRef.getSpecies()) )
                     {
                        listOfLocalReactions.add(reaction);
                        reactionAdded = true;
                     }
                 }                                                                          
              
             }
         
         }
       
         ArrayList listOfLocalParameters = new ArrayList();
         for (int j=0; j<listOfLocalReactions.size(); j++)                    // Get all the local parameters from each and
         {                                                                    // every reaction in a Module/Compartment and
             Reaction   reaction   = (Reaction)listOfLocalReactions.get(j);   // store in a temporary list array.
             KineticLaw kineticLaw = (KineticLaw) reaction.getKineticLaw();
           
             for (int k=0; k<kineticLaw.getListOfParametersSize(); k++)
             {
                 Parameter parameter = (Parameter)  kineticLaw.getListOfParameters().get(k);
                 listOfLocalParameters.add(parameter);
             }
         }
        
         if (listOfLocalParameters.size() != 0)
         {
            out.print("[");
            for (int j=0; j<listOfLocalParameters.size(); j++)
            {
                Parameter parameter = (Parameter) listOfLocalParameters.get(j);
                out.print( parameter.getId() );
                if (j < listOfLocalParameters.size()-1)
                {
                   out.print(", ");
                }
                else
                {
                   out.print(" : REAL]");
                }
            }
         } 
        
         out.println(" : MODULE =");

         out.println("\t" + "BEGIN");

         /* Fetch all the Global Parameters with Parameter-Ids different from the Parameters-Ids of Local Parameters. */   
         /* This is to ensure that Local Parameter declaration takes precedence over Global Parameter Declaration. So */
         /* if a Parameter with the same Id is declared in both Global and Local levels in SBML then the INPUT section*/
         /* in SAL MODULE will not contain the declaration correspoding to that Parameter.                            */
         
         boolean   isUniqueGlobalParameter;
         ArrayList listOfUniqueGlobalParameters = new ArrayList();

         for (int j=0; j<listOfGlobalParameters.size(); j++)
         {
             Parameter globalParameter = (Parameter) listOfGlobalParameters.get(j);
             isUniqueGlobalParameter   = true;

             for (int k=0; k<listOfLocalParameters.size(); k++)
             {
                 Parameter localParameter = (Parameter) listOfLocalParameters.get(k);
                 
                 if ( localParameter.getId().equals( globalParameter.getId() ) )
                 {
                    isUniqueGlobalParameter = false;
                    break;
                 }
             }
             
             if (isUniqueGlobalParameter)
             {
                listOfUniqueGlobalParameters.add(globalParameter);
             }    
         }
    
         if (listOfUniqueGlobalParameters.size() > 0)
         {
            out.print("\t" + "INPUT ");
         }

         for (int j=0; j<listOfUniqueGlobalParameters.size(); j++)
         {
             Parameter parameter = (Parameter) listOfUniqueGlobalParameters.get(j);
             out.print(parameter.getId());

             if (j < listOfUniqueGlobalParameters.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }
        
         out.print("\t" + "LOCAL ");
 
         for (int j=0; j<listOfLocalSpecies.size(); j++)
         {
             Species species = (Species)listOfLocalSpecies.get(j);
             out.print(species.getId());

             if (j < listOfLocalSpecies.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }

         out.print("\t" + "LOCAL ");
         ArrayList listOfSpeciesDot = new ArrayList();
         boolean generateDotVariable = false;
         
         for (int j=0; j<listOfLocalSpecies.size(); j++)               // Generates a dotVariable for the species in Local
         {                                                             // species list array if and only iff the constant
             Species species = (Species)listOfLocalSpecies.get(j);     // attribute for the species is "false".
             
             if ( species.getConstant().equals("true") )
             {
                generateDotVariable = false;
             }
             else if ( species.getBoundaryCondition().equals("true") )  // Generates a dotVariable for species in Local species
             {                                                          // list array even if the BoundaryCondition attribute is 
                  generateDotVariable = false;                          // "true", if in case there exists a rateRule for the 
                  for (int k=0; k<listOfRateRules.size(); k++)          // Variable.
                  {
                      Rule l = (Rule) listOfRateRules.get(k);
                      if ( l.getVariable().equals( species.getId() ) )
                      {
                         generateDotVariable = true;
                      }
                  }
           
             }
             else
             {
                  generateDotVariable = true;
             }
             
             if (generateDotVariable)
             {
                listOfSpeciesDot.add( species );
             }
         }

         for (int j=0; j<listOfSpeciesDot.size(); j++)
         {
             Species species = (Species)listOfSpeciesDot.get(j);
             out.print(species.getId() + "dot");

             if (j < listOfSpeciesDot.size()-1)
             {
                out.print(", ");
             }
             else
             {
                out.println(" : REAL");
             }
         }

        /** Get all the rules (Assignment/Algerbraic/Rate) which define the constraints on the Local Species into
            temporary list Arrays.
        */
        ArrayList listOfLocalAssignmentRules = new ArrayList();
        ArrayList listOfLocalAlgebraicRules  = new ArrayList();
        ArrayList listOfLocalRateRules       = new ArrayList();
        boolean   isLocalRule                = false;
        boolean   isLocalSpecies             = false;
                  ruleVariable               = "";

        for (int j=0; j<listOfAssignmentRules.size(); j++)
        {
            Rule assignmentRule = (Rule) listOfAssignmentRules.get(j);
            isLocalRule         = false;
          
            for (int k=0; k<assignmentRule.getListOfRuleVariablesSize(); k++)
            {
                isLocalSpecies  = false;
                ruleVariable    = (String) assignmentRule.getListOfRuleVariables().get(k); 
                for (int l=0; l<listOfLocalSpecies.size(); l++)
                {
                    Species localSpecies = (Species) listOfLocalSpecies.get(l);
                    if ( ruleVariable.equals( localSpecies.getId() ) )
                    {
                       isLocalSpecies = true;
                       break;
                    }
                }
                if (isLocalSpecies)
                {
                   isLocalRule = true;
                   break;
                }
            }
            
            if (isLocalRule)
            {
               listOfLocalAssignmentRules.add(assignmentRule);
            }
        } /* Finished getting AssignmentRules coressponding to the current Module into LocalAssignmentRules ArrayList */
      
        for (int j=0; j<listOfAlgebraicRules.size(); j++)
        {
            Rule algebraicRule  = (Rule) listOfAlgebraicRules.get(j);
            isLocalRule         = false;
          
            for (int k=0; k<algebraicRule.getListOfRuleVariablesSize(); k++)
            {
                isLocalSpecies  = false;
                ruleVariable    = (String) algebraicRule.getListOfRuleVariables().get(k); 
                for (int l=0; l<listOfLocalSpecies.size(); l++)
                {
                    Species localSpecies = (Species) listOfLocalSpecies.get(l);
                    if ( ruleVariable.equals( localSpecies.getId() ) )
                    {
                       isLocalSpecies = true;
                       break;
                    }
                }
                if (isLocalSpecies)
                {
                   isLocalRule = true;
                   break;
                }
            }
            
            if (isLocalRule)
            {
               listOfLocalAlgebraicRules.add(algebraicRule);
            }
        } /* Finished getting AlgebraicRules coressponding to the current Module into LocalAlgebraicRules ArrayList */

        for (int j=0; j<listOfRateRules.size(); j++)
        {
            Rule rateRule = (Rule) listOfRateRules.get(j);
            isLocalRule  = false;
          
            for (int k=0; k<rateRule.getListOfRuleVariablesSize(); k++)
            {
                isLocalSpecies = false;
                ruleVariable   = (String) rateRule.getListOfRuleVariables().get(k); 
                for (int l=0; l<listOfLocalSpecies.size(); l++)
                {
                    Species localSpecies = (Species) listOfLocalSpecies.get(l);
                    if ( ruleVariable.equals( localSpecies.getId() ) )
                    {
                       isLocalSpecies = true;
                       break;
                    }
                }
                if (isLocalSpecies)
                {
                   isLocalRule = true;
                   break;
                }
            }
            
            if (isLocalRule)
            {
               listOfLocalRateRules.add(rateRule);
            }
        } /* Finished getting RateRules coressponding to the current Module into LocalRateRules ArrayList */

        if ( listOfLocalParameters.size() > 0 || listOfLocalAssignmentRules.size() > 0 || listOfLocalAlgebraicRules.size() > 0 )
        {
           out.println("\tINVARIANT");                                                
           out.print("\t       ");

           for (int j=0; j<listOfLocalParameters.size(); j++)
           {
               Parameter parameter = (Parameter) listOfLocalParameters.get(j);
               out.print( parameter.getId() + " = " + parameter.getValue() );
               if (j < listOfLocalParameters.size()-1 || listOfLocalAssignmentRules.size() > 0 ||
                       listOfLocalAlgebraicRules.size() > 0 )
               {
                  out.print(" AND ");
               }
               else
               {
                  out.println("");
               }
           }

           for (int j=0; j<listOfLocalAssignmentRules.size(); j++)
           {
               Rule localAssignmentRule = (Rule) listOfLocalAssignmentRules.get(j);

               out.print( localAssignmentRule.getVariable() + " = " + localAssignmentRule.getExpression() );

               if ( j < listOfLocalAssignmentRules.size()-1 || listOfLocalAlgebraicRules.size() > 0 )
               {
                  out.println(" AND");
               }
               else
               {
                  out.println();
               }
           }

           for (int j=0; j<listOfLocalAlgebraicRules.size(); j++)
           {
               Rule localAlgebraicRule = (Rule) listOfLocalAlgebraicRules.get(j);

               out.print( localAlgebraicRule.getExpression() + " = 0" );

               if ( j < listOfLocalAlgebraicRules.size()-1 )
               {
                  out.println(" AND");
               }
               else
               {
                  out.println();
               }
           }

        } /* Finished writing INVARIANT section for the current Module */ 

        if (listOfLocalSpecies.size() != 0)
        {
           out.println("\tINITIALIZATION");
           out.print("\t       ");

           for (int j=0; j<listOfLocalSpecies.size(); j++)
           {
               Species localSpecies = (Species) listOfLocalSpecies.get(j);
               out.print( localSpecies.getId() + " = " + localSpecies.getInitialAmount() );
               if ( j < listOfLocalSpecies.size()-1 ) 
               {
                  out.print("; ");
               }
               else 
               {
                  out.println("");
               }
           }

        } /* Finished writing INITIALIZATION section for the current Module */

        ArrayList listOfLocalEvents = new ArrayList();
        boolean   isLocalEvent = false;
        String    eventVariable = "";

        /** Get all the events which belong to the Compartment/Module */
        for (int j=0; j<listOfEvents.size(); j++)
        {
            Event localEvent = (Event) listOfEvents.get(j);
           
            for (int k=0; k<localEvent.getListOfEventVariablesSize(); k++)
            {
                isLocalSpecies = false;
                eventVariable  = (String) localEvent.getListOfEventVariables().get(k);
                for (int l=0; l<listOfLocalSpecies.size(); l++)
                {
                    Species localSpecies = (Species) listOfLocalSpecies.get(l);
                    if ( eventVariable.equals( localSpecies.getId() ) )
                    {
                       isLocalSpecies = true;
                       break;
                    }
                }
                if (isLocalSpecies)
                {
                   isLocalEvent = true;
                   break;
                }
            }
            
            if (isLocalEvent)
            {
               listOfLocalEvents.add(localEvent);
            }
        }        
         
        if (listOfLocalSpecies.size() != 0)
        {
            boolean rateRuleVariable = false;
            out.println("\tTRANSITION");
           
            if (listOfLocalEvents.size() !=0)
            { 
               out.println("\t       [");

               for (int j=0; j<listOfLocalEvents.size(); j++ )
               {
                   Event localEvent = (Event) listOfLocalEvents.get(j);
                   out.print( "\t       " + localEvent.getTriggerExpression() );
                   out.println("-->");
                   
                   for (int k=0; k<localEvent.getListOfEventAssignmentsSize(); k++)
                   {
                       out.print( "\t\t       " + localEvent.getListOfEventAssignments().get(k) ); 
              
                       if (k < localEvent.getListOfEventAssignmentsSize() - 1)
                       {
                          out.println(";");
                       }
                       else
                       {
                          out.println();
                       }
                   }
                  
                   if (j < listOfLocalEvents.size()-1)
                   {
                      out.println("\t       []");
                   }
                   else
                   {
                      out.println("\t       ];");
                   }
               }
            }

           /* This block writes the rate reactions determining the change in species concentration.      */
           /* For each dotSpecies we need to check whether its rate is determined by Rate Rule before    */
           /* generating a differential equation for that species. If the rate is not determined by      */
           /* Rate Rule then build a differential equation for the species with help of Kinetic formulas */
           /* of the Reactions in which the species appears. If the species appears in list of Reactants */
           /* then place a '-' sign before the Kinetic formula else if the species appears in list of    */
           /* products then place '+' sign before the Kinetic formula.                                   */

           for (int j=0; j<listOfSpeciesDot.size(); j++)
           {
               Species dotSpecies       = (Species) listOfSpeciesDot.get(j);
                       rateRuleVariable = false;

               for (int l=0; l<listOfLocalRateRules.size(); l++)
               {
                   Rule localRateRule = (Rule) listOfLocalRateRules.get(l);
                   if ( dotSpecies.getId().equals( localRateRule.getVariable() ) )
                   {
                      rateRuleVariable = true;
                      out.print("\t       ");
                      out.print( localRateRule.getVariable() + "dot' = " + localRateRule.getExpression() );

                      if ( j < listOfSpeciesDot.size()-1 )
                      {
                         out.println(" ;");
                      }
                      else
                      {
                         out.println();
                      }

                      break;
                   }
               } /* Finished writing all the Rate Rules */
               
               if (!rateRuleVariable)                              // Generate differential equations for species
               {                                                   // whose rate change is not determined by Rate Rule.
                  out.print("\t       ");
                  out.print( dotSpecies.getId() + "dot'" + " = " );
                  boolean firstProductExpression = true;

                  for (int k=0; k<listOfLocalReactions.size(); k++)
                  {
                      Reaction localReaction = (Reaction)   listOfLocalReactions.get(k);
                      KineticLaw kineticLaw  = (KineticLaw) localReaction.getKineticLaw();
                   
                      for (int l=0; l<localReaction.getListOfReactantsSize();l++)
                      {
                  
                          SpeciesReference speciesRef = (SpeciesReference) localReaction.getListOfReactants().get(l);
                   
                          if (dotSpecies.getId().equals( speciesRef.getSpecies() ) )
                          {                                                                  
                             out.print( " - " + speciesRef.getStoichiometry() + "*" + kineticLaw.getFormula() );
                                                                                      // If a dotSpecies appears in list of
                                                                                      // reactants, then apply a negative   
                             firstProductExpression = false;                          // sign to kinetic formula.
                             break;
                          }
                   
                      }
                  
                      for (int l=0; l<localReaction.getListOfProductsSize();l++)
                      {
                          SpeciesReference speciesRef = (SpeciesReference) localReaction.getListOfProducts().get(l);
                     
                          if ( dotSpecies.getId().equals( speciesRef.getSpecies() ) )
                          {
                             if (firstProductExpression)                            // Do not print '+' before kineticFormula
                             {                                                      // if the kineticFormula is the first one. 
                                out.print( " " + speciesRef.getStoichiometry() + "*" + kineticLaw.getFormula() );          
                                firstProductExpression = false;
                             }
                             else
                             {
                                out.print( " + " + speciesRef.getStoichiometry() + "*" + kineticLaw.getFormula() );
                             }
                             break;
                          }
                       }
                  }
                  
                  if ( j < listOfSpeciesDot.size()-1 )
                  {
                     out.println(" ;");
                  }
                  else
                  {
                     out.println();
                  }
  
               }
           }

           out.println("\tEND;");
        }

        /* Clear all the List arrays used to store data coressponding to a local Module. */
         
        listOfLocalParameters.clear();
        listOfLocalSpecies.clear();              
        listOfLocalReactions.clear();           
        listOfSpeciesDot.clear();           
        listOfLocalAssignmentRules.clear();           
        listOfLocalAlgebraicRules.clear();              
        listOfLocalRateRules.clear();                   
        listOfConstantGlobalParameters.clear();              
        listOfNonConstantGlobalParameters.clear();            
        listOfUniqueGlobalParameters.clear();
        listOfLocalEvents.clear();            
          
     } /* Compartment Loop ends here */

     out.println("END");
     System.out.println("... Finished Writing as Well");
  }

  /**
   * This method catches <em>start of element<em> event.
   * @throws SAXException
   */
  public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
  {
    
    if (qName.equals("model"))
    {
       modelId = atts.getValue("id");
    }
    
    if (qName.equals("functionDefinition"))
    {
       functionId           = atts.getValue("id");
       isFunctionDefinition = true;
    }

    if (isFunctionDefinition)
    {
       if (isMath)
       {
          if (qName.equals("bvar"))
          {
             isFunctionVariable = true;
          }
       }
    }

    if (qName.equals("compartment"))
    { 
       listOfCompartments.add(new Compartment(atts.getValue("id"),atts.getValue("spatialDimensions")));
    }
    
    if (qName.equals("species"))
    {
       String speciesBoundaryCondition  = "false"; 
       String speciesConstant           = "false";
       String speciesInitialQuantity    = "";
      
       if (atts.getValue("constant") != null)
       {
          speciesConstant = atts.getValue("constant");
       }
       
       if (atts.getValue("boundaryCondition") != null)
       {
          speciesBoundaryCondition = atts.getValue("boundaryCondition");
       }

       if (atts.getValue("initialAmount") != null)                 // Currently I'am treating initialAmount and Initial Conc. in
       {                                                           // the same manner as there is no unit definition structure.
          speciesInitialQuantity = atts.getValue("initialAmount"); // This blocks needs to be changed if unit definition structure
          speciesInitialQuantity = parseExpDecimal(speciesInitialQuantity); //added to SAL in future versions.
       }
       else if (atts.getValue("initialConcentration") != null)
       {
          speciesInitialQuantity = atts.getValue("initialConcentration");
          speciesInitialQuantity = parseExpDecimal(speciesInitialQuantity);
       }

       listOfSpecies.add( new Species(atts.getValue("id"), speciesInitialQuantity, atts.getValue("compartment"),
                                      speciesBoundaryCondition, speciesConstant ));
    }

    if (qName.equals("assignmentRule"))
    {
       ruleVariable     = atts.getValue("variable");
       listOfRuleVariables.add(ruleVariable);
       isAssignmentRule = true;
    }

    if (qName.equals("algebraicRule"))
    {
       ruleVariable    = "0";
       isAlgebraicRule = true;
    }

    if (qName.equals("rateRule"))
    {
       ruleVariable = atts.getValue("variable");
       listOfRuleVariables.add(ruleVariable);
       isRateRule   = true;
    }

    if ( (isAssignmentRule)     || (isAlgebraicRule) || (isRateRule) ||
         (isFunctionDefinition) || (isTrigger)       || (isEventAssignment) )
    {
       if (isMath)
       {
          if (qName.equals("apply"))
          {
             formula = formula + " (";
          }
          else if( qName.equals("ci") || qName.equals("cn") || qName.equals("csymbol") )
          {
             mathIdentifier.setLength(0);
             isMathIdentifier = true;
          } 
       }
    } /* This block reads the MathML in each structure and constructs a PreFixExpression out of it */
   
    if (qName.equals("reaction"))
    {
       reactionName       = atts.getValue("id");
       reactionReversible = atts.getValue("reversible");
    }
    
    if (qName.equals("listOfReactants"))
    {
       isListOfReactants = true;
    }
    
    if (qName.equals("listOfProducts"))
    {
       isListOfProducts = true;
    }

    if (qName.equals("listOfModifiers"))
    {
       isListOfModifiers = true;
    }

    if (qName.equals("speciesReference"))
    {
       speciesValue       = atts.getValue("species");
       stoichiometryValue = "1"; 
       isSpeciesReference = true;
    }
    
    if (qName.equals("modifierSpeciesReference"))
    {
       if(isListOfModifiers)
       {
          listOfModifiers.add(new ModifierSpeciesReference(atts.getValue("species")));
       }
    }

    if (qName.equals("math"))
    {
       isMath = true;
    }

    if (isSpeciesReference)
    {
       if (isMath)
       {
          if (qName.equals("cn"))
          {
             mathIdentifier.setLength(0);
             isMathIdentifier = true;
          }
       }
    }
    
    if (qName.equals("kineticLaw"))
    {
       isKineticLaw = true;
//change#01
       formula = atts.getValue("formula");
    }

    if (isKineticLaw)
    { 
       if (isMath)
       {
          if (qName.equals("apply"))
          {
             formula = formula + " (";
          }
          else if( qName.equals("ci") || qName.equals("cn") || qName.equals("csymbol") )
          {
             mathIdentifier.setLength(0);
             isMathIdentifier = true;
          }
       }
       if (qName.equals("parameter"))
       {
          String parameterValue = parseExpDecimal(atts.getValue("value"));
          listOfParameters.add(new Parameter(atts.getValue("id"),parameterValue));
       }
    }
    
    if ( (!isKineticLaw) && qName.equals("parameter") )
    {
       String parameterValue = parseExpDecimal(atts.getValue("value"));
       listOfGlobalParameters.add(new Parameter(atts.getValue("id"),parameterValue));
    }

    if (qName.equals("trigger"))
    {
       isTrigger = true;
    }

    if (qName.equals("eventAssignment"))
    {
       isEventAssignment = true;
       eventVariable     = atts.getValue("variable");
       listOfEventVariables.add(eventVariable);
    }

  }
 
  /**
   * This method catches <em>end of element<em> event.
   * @throws SAXException
   */
  public void endElement(String uri, String localName, String qName) throws SAXException
  {
   
    if (qName.equals("assignmentRule")) 
    {
       formula          = Prefix_To_Infix_Converter(formula);
       listOfAssignmentRules.add( new Rule(ruleVariable, formula, listOfRuleVariables) );
       formula          = "";
       isAssignmentRule = false;
       listOfRuleVariables.clear();
    }  /* Converts the AssignmentRule from the Prefix form to Infix form and adds it to ArrayList */

    if (qName.equals("algebraicRule"))
    {
       formula         = Prefix_To_Infix_Converter(formula);
       listOfAlgebraicRules.add( new Rule(ruleVariable, formula, listOfRuleVariables) );
       formula         = "";
       isAlgebraicRule = false;
       listOfRuleVariables.clear();
    }  /* Converts the AlgebraicRule from the Prefix form to Infix form and adds it to ArrayList */

    if (qName.equals("rateRule"))
    {
       formula    = Prefix_To_Infix_Converter(formula);
       listOfRateRules.add( new Rule(ruleVariable, formula, listOfRuleVariables) );
       formula    = "";
       isRateRule = false;
       listOfRuleVariables.clear();
    }  /* Converts the rateRule from the Prefix form to Infix form and adds it to ArrayList */
   
    if (isSpeciesReference)
    {
       if (isMath)
       {
          if ( qName.equals("ci") || qName.equals("cn") )
          {
             isMathIdentifier   = false;
             stoichiometryValue = "";
             stoichiometryValue = stoichiometryValue + mathIdentifier;
             stoichiometryValue = stoichiometryValue.trim();
             stoichiometryValue = parseExpDecimal(stoichiometryValue);
          }
       }
       
    }
       
    if (qName.equals("speciesReference"))
    {
       isSpeciesReference = false; 
 
       if (isListOfReactants)
       {
          listOfReactants.add(new SpeciesReference(speciesValue,stoichiometryValue));
       }
       
       if (isListOfProducts)
       {
          listOfProducts.add(new SpeciesReference(speciesValue,stoichiometryValue));
       }

    }

    if (qName.equals("listOfReactants"))
    {
       isListOfReactants = false;
    }

    if (qName.equals("listOfProducts"))
    {
       isListOfProducts = false;
    }
   
    if (qName.equals("listOfModifiers"))
    {
       isListOfModifiers = false;
    }

    if (qName.equals("kineticLaw"))
    {
//change#02       formula      = Prefix_To_Infix_Converter(formula);
       isKineticLaw = false;
    }

    if ( (isTrigger) || (isEventAssignment) )
    {
       if (qName.equals("ci"))                                            // This block keeps track of all the variables
       {                                                                  // involved in Event Assignments;        
          String Variable = "";
          Variable = Variable + mathIdentifier;
          Variable = Variable.trim();
          listOfEventVariables.add(Variable);
       }
    }
   
    if ( (isKineticLaw) || (isAssignmentRule) || (isAlgebraicRule)   ||
         (isRateRule)   || (isTrigger)        || (isEventAssignment) ||
         (isFunctionDefinition && !isFunctionVariable) )
    {
       if (isMath)
       {
          
          if (qName.equals("ci"))
          {                                                                 
             if ( (isRateRule) || (isAssignmentRule) || (isAlgebraicRule) )         // This block keeps track of all the variables
             {                                                                      // involved in different types of rules;
                String Variable = "";
                Variable = Variable + mathIdentifier;
                Variable = Variable.trim();
                listOfRuleVariables.add(Variable);
             }
          }

          if ( qName.equals("ci") || qName.equals("cn") || qName.equals("csymbol") ) // This block parsers the mathML expressions
          {
             isMathIdentifier = false;
             String Identifier = "";
             Identifier = Identifier + mathIdentifier;
             Identifier = Identifier.trim();
             
             if (qName.equals("cn"))
             {
                Identifier = parseExpDecimalPreFix(Identifier);                     // Convert the numbers to Power10-PreFix
                                                                                    // format. Which will be converted back to
                                                                                    // InFixFormat later.
             }            
              
             formula          = formula + " " + Identifier;
          }  
          if (qName.equals("apply"))
          {
             formula = formula + " )";
          }
          if (qName.equals("times"))
          {
             formula = formula + " *";
          }
          if (qName.equals("divide"))
          {
             formula = formula + " /";
          }
          if (qName.equals("plus"))
          {
             formula = formula + " +";
          }
          if (qName.equals("minus"))
          {
             formula = formula + " -";
          }
          if (qName.equals("power"))
          {
             formula = formula + " ^";
          }
          if (qName.equals("eq"))
          {
             formula = formula + " ==";
          }
          if (qName.equals("neq"))
          {
             formula = formula + " !=";
          }
          if (qName.equals("gt"))
          {
             formula = formula + " >";
          }
          if (qName.equals("leq"))
          {
             formula = formula + " >=";
          }
          if (qName.equals("lt"))
          {
             formula = formula + " <";
          }
          if (qName.equals("leq"))
          {
             formula = formula + " <=";
          }
       }
    }
    
    if (isFunctionDefinition && isFunctionVariable)
    {  
       if (isMath)
       {
          if ( qName.equals("ci") )
          {
             isMathIdentifier = false;
             functionVariable = "";
             functionVariable = functionVariable + mathIdentifier;
             functionVariable = functionVariable.trim();
             listOfFunctionVariables.add( functionVariable );
          }  
       } 
    }
    
    if (qName.equals("functionDefinition"))
    {
       isFunctionDefinition = false;
       formula              = Prefix_To_Infix_Converter(formula);
       listOfFunctionDefinitions.add( new FunctionDefinition(functionId,listOfFunctionVariables,formula) );
       listOfFunctionVariables.clear();
       formula              = ""; 
    }
       
    if (qName.equals("bvar"))
    {
       isFunctionVariable   = false;
    }

    if (qName.equals("trigger"))
    {  
       triggerExpression    = "";
       triggerExpression    = Prefix_To_Infix_Converter(formula);
       formula              = "";
       isTrigger            = false;
    }

    if (qName.equals("eventAssignment"))
    {
       formula           = Prefix_To_Infix_Converter(formula);
       eventAssignment   = eventVariable + " = " + formula;
       listOfEventAssignments.add(eventAssignment);
       formula           = "";
       isEventAssignment = false;
    }

    if (qName.equals("event"))
    {
       listOfEvents.add( new Event(triggerExpression, listOfEventAssignments, listOfEventVariables) );
       listOfEventAssignments.clear();
       listOfEventVariables.clear();
    }

    if (qName.equals("math"))
    {
       isMath = false;
    }

    if (qName.equals("reaction"))
    {
        Reaction r   = new Reaction(reactionName,reactionReversible);
        r.cloneListOfReactants(listOfReactants);
        r.cloneListOfProducts(listOfProducts);
        r.cloneListOfModifiers(listOfModifiers);
        KineticLaw k = new KineticLaw(formula);
        k.cloneListOfParameters(listOfParameters);
        r.cloneKineticLaw(k);

        listOfReactants.clear();
        listOfProducts.clear();
        listOfModifiers.clear();
        listOfParameters.clear();
        formula = "";

        listOfReactions.add(r);
    }
       
  }
 
  public void characters(char[] chars, int start, int len) throws SAXException
  {
    if (isMathIdentifier)
    {
       mathIdentifier.append(chars, start, len);
    }
  }

  /** Converts a Prefix Expression to Infix Expression
   *  Note: The tokens in the input string should be separated by blank space.
   *  @return InfixExpression
   */
  public String Prefix_To_Infix_Converter (String prefixExpression)
  {
    Stack expressionStack = new Stack();
    StringTokenizer t     = new StringTokenizer(prefixExpression);
    String operator;
 
    do
    {
      Object element = new String(t.nextToken());
 
      if ( element.equals(")") )
      {
         Stack tempTermStack = new Stack();
         element             = expressionStack.pop();
 
         while ( !element.equals("(") )
         {
           tempTermStack.push(element);
           element = (String) expressionStack.pop();
         }
 
         Object term = new String("(");
         operator    = (String)tempTermStack.pop();

         if ( operator.equals("*")  || operator.equals("/")  || operator.equals("+")  ||
              operator.equals("-")  || operator.equals("^")  || operator.equals(">")  ||
              operator.equals("<")  || operator.equals("<=") || operator.equals(">=") ||
              operator.equals("!=") || operator.equals("==") )
         {  
            while (!tempTermStack.empty())
            {
              term = term + (String)tempTermStack.pop();
             
              if (!tempTermStack.empty())
              {
                 term = term + operator;
              }
           
            }
         }   
         else
         { 
            term = operator + term;             // In this case operator refers to function name. Also
                                                // operator is already initialized to left paranthesis
                                                // so we can concatenate function name and "(" and assign
                                                // it back to term.
            while (!tempTermStack.empty())
            {
              term = term + (String)tempTermStack.pop();
              
              if (!tempTermStack.empty())
              {
                 term = term + ",";
              }
            }
         }

         term = term + ")";
         expressionStack.push(term);
 
       }
       else
       {
         expressionStack.push(element);
       }

     }while(t.hasMoreTokens());
 
     String expression = (String)expressionStack.pop();
     return expression;
  }

  /** This method converts a String (numbers) containing decimals and 'e' to an equivalent expression in powers of 10. 
   *  For example 2.345e-2 will be converted to 2345 * (1/1000) * (1/100)
   *  Note: This method inturn makes a call to parseDecimal method to convert Strings containing Decimal numbers to
   *        an equivalent expression in powers of 10.
   *  @return powerTenExpression
   */
  public String parseExpDecimal(String oldString)
  {
     int posOfExp     = oldString.indexOf('e');
     if (posOfExp >= 0)
     {
        if (oldString.charAt(posOfExp+1) == '-')
        {
           String beginStr  = oldString.substring(0,posOfExp);
           beginStr         = decimalParser(beginStr);
           String endStr    = oldString.substring(posOfExp+2,oldString.length());
           String tenPower  = "1";
           for (int i=0; i < Integer.parseInt(endStr); i++)
           {
               tenPower = tenPower + "0";
           }
           String newString = beginStr + " * (1/" + tenPower + ")";
           return newString;
        }
        else
        {
           String beginStr  = oldString.substring(0,posOfExp);
           beginStr         = decimalParser(beginStr);
           String endStr    = oldString.substring(posOfExp+1,oldString.length());
           String tenPower  = "1";
           for (int i=0; i < Integer.parseInt(endStr); i++)
           {
               tenPower = tenPower + "0";
           }
           String newString = beginStr + " * (" + tenPower + ")";
           return newString;
          
        }
     }
     else
     {
        String newString = decimalParser(oldString);
        return newString;
     }  
  }  
 
  /** This method converts Strings containing Decimal numbers to an equivalent expression in powers of 10.
   *  For Example 2.345 will be converted to 2345 * (1/1000)      
   */
  public String decimalParser(String oldString)
  {
     int posOfDecimal =  oldString.indexOf('.');

     if (posOfDecimal >= 0)
     {
        int strLength    = oldString.length();
        String beginStr  = oldString.substring(0,posOfDecimal);
        String endStr    = oldString.substring(posOfDecimal+1,strLength);
        String tenPower  = "1";
        for (int i=0; i < endStr.length(); i++)
        {
            tenPower = tenPower + "0";
        }
        String newString = beginStr + endStr + " * (1/" + tenPower + ")";
        return newString;
     }
     else
     {
        return oldString;
     }
  }

  /** This method converts a String (numbers) containing decimals and 'e' to an equivalent expression in powers of 10. 
   *  For example 2.345e-2 will be converted to ( * 2345 ( / 1 1000 ) ( / 1 100 ) ) 
   *  Note: This method inturn makes a call to parseDecimal method to convert Strings containing Decimal numbers to
   *        an equivalent expression in powers of 10.
   *  @return powerTenExpression
   */
  public static String parseExpDecimalPreFix(String oldString)
  {
     int posOfExp     = oldString.indexOf('e');
     if (posOfExp >= 0)
     {
        if (oldString.charAt(posOfExp+1) == '-')
        {
           String beginStr  = oldString.substring(0,posOfExp);
           beginStr         = decimalParserPreFix(beginStr);
           String endStr    = oldString.substring(posOfExp+2,oldString.length());
           String tenPower  = "1";
           for (int i=0; i < Integer.parseInt(endStr); i++)
           {
               tenPower = tenPower + "0";
           }
           String newString = "( * " + beginStr + " ( / 1 " + tenPower + " ) " + ")";
           return newString;
        }
        else
        {
           String beginStr  = oldString.substring(0,posOfExp);
           beginStr         = decimalParserPreFix(beginStr);
           String endStr    = oldString.substring(posOfExp+1,oldString.length());
           String tenPower  = "1";
           for (int i=0; i < Integer.parseInt(endStr); i++)
           {
               tenPower = tenPower + "0";
           }
           String newString = "( * " + beginStr + " ( " + tenPower + " ) " + ")";
           return newString;
          
        }
     }
     else
     {
        String newString = decimalParserPreFix(oldString);
        if (newString.equals(oldString))
        {
           return oldString;
        }
        else
        {
           return "( * " + newString + " )"; 
        }
     }  
  }  
 
  /** This method converts Strings containing Decimal numbers to an equivalent expression in powers of 10.
   *  For Example 2.345 will be converted to 2345 ( / 1 1000 )       
   */
  public static String decimalParserPreFix(String oldString)
  {
     int posOfDecimal =  oldString.indexOf('.');

     if (posOfDecimal >= 0)
     {
        int strLength    = oldString.length();
        String beginStr  = oldString.substring(0,posOfDecimal);
        String endStr    = oldString.substring(posOfDecimal+1,strLength);
        String tenPower  = "1";
        for (int i=0; i < endStr.length(); i++)
        {
            tenPower = tenPower + "0";
        }
        String newString = beginStr + endStr + " ( / 1 " + tenPower + " )";
        return newString;
     }
     else
     {
        return oldString;
     }
  }

  /**
   * This method is used to check whether the document is read properly in the corresponding fields or not.
   * This method "can" be removed once the program is completed.
   */
  public void printCompartments()
  {
    System.out.println("Model Id: " + modelId);


    for (int j=0; j<listOfFunctionDefinitions.size(); j++)
    {
        FunctionDefinition f = (FunctionDefinition)  listOfFunctionDefinitions.get(j);
        System.out.println("Function Definition Id:" + f.getFunctionId() + " FunctionDefinitionExpression: "
                                                     + f.getFunctionExpression());
        for (int i=0; i<f.getListOfFunctionVariablesSize(); i++)
        {
            System.out.println( "variable" + i + ":" + f.getListOfFunctionVariables().get(i) );
        }
    }

    for (int j=0; j<listOfGlobalParameters.size(); j++)
    {
        Parameter p = (Parameter)  listOfGlobalParameters.get(j);
        System.out.println("Parameter-Id: " + p.getId() + "Value: " + p.getValue());
    }

    for(int i=0; i<listOfCompartments.size(); i++)
    {
       Compartment c = (Compartment)listOfCompartments.get(i);
       System.out.println("Compartment Id: " + c.getId()
                          + ", spatialDimensions: " + c.getSpatialDimensions());
    }
    
    for(int i=0; i<listOfSpecies.size(); i++)
    {
       Species s = (Species)listOfSpecies.get(i);
       System.out.println("Species Id: " + s.getId() + ", InitialAmount: " + s.getInitialAmount() + ", Compartment: "
                          + s.getCompartment() + ", boundaryCondition: " + s.getBoundaryCondition() + ", Constant: "
                          + s.getConstant());
    }

    for (int j=0; j<listOfAssignmentRules.size(); j++)
    {
        Rule l = (Rule) listOfAssignmentRules.get(j);
        System.out.println("Assignment-RHS-Expression: " + l.getExpression() + "Variable: " + l.getVariable());
   
        System.out.println("list of Assignment Rule Variables: ");
        for (int i=0; i<l.getListOfRuleVariablesSize(); i++)
        {
            System.out.print(l.getListOfRuleVariables().get(i) + " ");
        }
        System.out.println(); 
    }
      
    for (int j=0; j<listOfAlgebraicRules.size(); j++)
    {
        Rule l = (Rule) listOfAlgebraicRules.get(j);
        System.out.println("Algebraic-RHS-Expression: " + l.getExpression() + "Variable: " + l.getVariable());

        System.out.println("list of Algebraic Rule Variables: ");
        for (int i=0; i<l.getListOfRuleVariablesSize(); i++)
        {
            System.out.print(l.getListOfRuleVariables().get(i) + " ");
        }
        System.out.println(); 
             
    }

    for (int j=0; j<listOfRateRules.size(); j++)
    {
        Rule l = (Rule) listOfRateRules.get(j);
        System.out.println("Rate-RHS-Expression: " + l.getExpression() + "Variable: " + l.getVariable());

        System.out.println("list of Rate Rule Variables: ");
        for (int i=0; i<l.getListOfRuleVariablesSize(); i++)
        {
            System.out.print(l.getListOfRuleVariables().get(i) + " ");
        }
        System.out.println(); 
             
    }

    for(int i=0; i<listOfReactions.size();i++)
    {

       Reaction r = (Reaction)listOfReactions.get(i);
       System.out.println("Reaction Id: " + r.getId() + ", Reversible: " +  r.getReversible());
       
       for (int j=0; j<r.getListOfReactantsSize();j++)
       {
           SpeciesReference s = (SpeciesReference) r.getListOfReactants().get(j);
           System.out.println("Reactant Id: " +  s.getSpecies() + ", Stoichiometry: " + s.getStoichiometry());
       }

       for (int j=0; j<r.getListOfProductsSize();j++)
       {
           SpeciesReference s = (SpeciesReference) r.getListOfProducts().get(j);
           System.out.println("Product Id: " +  s.getSpecies() + ", Stoichiometry: " + s.getStoichiometry());
       }
       
       for (int j=0; j<r.getListOfModifiersSize();j++)
       {
           ModifierSpeciesReference m = (ModifierSpeciesReference) r.getListOfModifiers().get(j);
           System.out.println("Modifier Id: " +  m.getSpecies());
       }

       KineticLaw k = (KineticLaw) r.getKineticLaw();
       System.out.println("Kinetic Law-Formula: " + k.getFormula());
       
       for (int j=0; j<k.getListOfParametersSize(); j++)
       {
           Parameter p = (Parameter)  k.getListOfParameters().get(j);
           System.out.println("Parameter-Id: " + p.getId() + "Value: " + p.getValue());
       }
       
    }

    for (int j=0; j<listOfEvents.size(); j++)
    {
        Event e = (Event) listOfEvents.get(j);
        System.out.println("Trigger:" + e.getTriggerExpression()); 
        for (int i=0; i<e.getListOfEventAssignmentsSize(); i++)
        {
            System.out.println( "Event Assignment" + i + ":" + e.getListOfEventAssignments().get(i) );
        }
        for (int i=0; i<e.getListOfEventVariablesSize(); i++)
        {
            System.out.println( "Event Variable"   + i + ":" + e.getListOfEventVariables().get(i) );
        }
    }

  }

}
