package sbmlClasses;
import java.util.*;
/**
 * A <code>Ruler</code> object represents the constraints on variables for cases in which the constraints cannot be 
 * expressed using reactions. There are three different kinds of rules - Algebraic, Assignment and Rate rules which
 * are classified based on their functional forms.
 */
public class Rule
{
  public Rule(String variable, String expression, ArrayList listOfRuleVariables)
  {
    this.variable   = variable;
    this.expression = expression;
    this.listOfRuleVariables  = (ArrayList) listOfRuleVariables.clone();
  }
  
  /** Gets the variable for which the constraint is defined
   *  @return variable
   */
  public String getVariable()
  {
    return this.variable;
  }
  
  /** Gets the expression which defines the constraint placed on the variable
   *  @return expression
   */
  public String getExpression()
  {
    return this.expression;
  }

  /** Gets the number of variables which are involved in defining the constraint.
   *  This number includes the main variable on which the constraint is defined using the remaining variable.
   *  @return size_of_RuleVariables_ArrayList
   */
  public int getListOfRuleVariablesSize()
  {
    return listOfRuleVariables.size();
  }

  /** Gets the list array containing the variables which are used to define the constraint/rule.
   *  This list includes the main variable on which the constraint is defined using the remaining variable.
   *  @return RuleVariables_ArrayList
   */
  public ArrayList getListOfRuleVariables()
  {
    return this.listOfRuleVariables;
  }

  private ArrayList listOfRuleVariables;
  private String    variable;
  private String    expression;
}
