package sbmlClasses;
/** 
 * A <code>Species</code> object refers to chemical entities that take part in reactions.
 * These include simple ions (e.g., protons, calcium), simple molecules (e.g., glucose,
 * ATP), large molecules (e.g., RNA, polysaccharides, and proteins), and others.
 */
public class Species
{
  public Species(String id, String initialAmount, String compartment, String boundaryCondition, String constant)
  {
    this.id                = id;
    this.initialAmount     = initialAmount;
    this.compartment       = compartment;
    this.boundaryCondition = boundaryCondition;
    this.constant          = constant;
  }

  public String getId()
  {
    return id;
  }
  
  /** Gets the initial quantity of the corresponding species in the compartment.
   *  @return initialAmount
   */
  public String getInitialAmount()
  {
    return initialAmount;
  }
  
  /** Gets the compartment of the species in which the species is located.
   *  @return compartment
   */
  public String getCompartment()
  {
    return compartment;
  }
  
  /** Boundary condition of a Species indicates whether the species is on the bounday of the reaction or not.
   *  If the boundaryCondition is "true" then the species' concentration is not determined by the set of reactions
   *  even when a species appears as a product or reactant in a reaction system. This means a differential equation
   *  governing the change in concentration of the species should not be generated.
   *  @return boundaryCondition 
   */
  public String getBoundaryCondition()
  {
    return boundaryCondition;
  }

  /** Constant field of a Species indicates whether the species concentration can vary during simulation or not.
   *  If the Constant field is "true" then the species' concentration never changes during simulation.
   *  @return constant
   */
  public String getConstant()
  {
    return constant;
  }

  private String id;
  private String initialAmount;
  private String compartment;
  private String boundaryCondition;
  private String constant;
}
